#include "local.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>

/*

  Copyright (c) 2024 Juha Railoma

  Permission to use, copy, modify, and/or distribute this software for
  any purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
  WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
  OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
  FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
  DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
  OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

/* statistics */
int64_t stat_blk_count;
int64_t stat_chr_count;
int64_t stat_dir_count;
int64_t stat_lnk_count;
int64_t stat_fifo_count;
int64_t stat_file_count;
int64_t stat_sock_count;
int64_t stat_unknown_count;
int64_t stat_dir_open_failed;
int64_t stat_dir_stat_failed;
int64_t stat_file_stat_failed;

int64_t stat_origin_files;
int64_t stat_origin_bytes;

int64_t stat_linked_files;
int64_t stat_linked_bytes;
int64_t stat_linked_blocks;

/* common variables, unstructed to speed up */
struct obstack               *obs;
const char                   *app_progname;

int                           app_quiet;
int                           app_debug;
int                           app_verbose;

int                           app_dry_run;
int                           app_keep_going;
int                           app_content_only;

int                           app_fds_max;
int                           app_fds_safe;
int                           app_path_max;
int                           app_path_safe;
int                           app_nlinks_max;
int                           app_nlinks_limit;

int                           app_dirs_count;
char                        **app_dirs;

int                           app_sizes_count;
struct z_sizes_t             *app_sizes[Z_SIZES_HASH];

static void
__logn(const char *format, va_list ap) {
    vfprintf(stdout, format, ap);
}

static void
__log(const char *format, va_list ap) {
    __logn(format, ap);
    fputs("\n", stdout);
}

int
log_debug(const char *format, ...) {

    if (app_debug) {
        va_list ap;

        va_start(ap, format);
        __log(format, ap);
        va_end(ap);
    }

    return 0;
}

int
logn_debug(const char *format, ...) {

    if (app_debug) {
        va_list ap;

        va_start(ap, format);
        __logn(format, ap);
        va_end(ap);
    }

    return 0;
}

int
log_print(const char *format, ...) {

    if (!app_quiet) {
        va_list ap;

        va_start(ap, format);
        __log(format, ap);
        va_end(ap);
    }

    return 0;
}

int
logn_print(const char *format, ...) {

    if (!app_quiet) {
        va_list ap;

        va_start(ap, format);
        __logn(format, ap);
        va_end(ap);
    }

    return 0;
}

int
log_verbose(const char *format, ...) {

    if (app_verbose) {
        va_list ap;

        va_start(ap, format);
        __log(format, ap);
        va_end(ap);
    }

    return 0;
}

int
logn_verbose(const char *format, ...) {

    if (app_verbose) {
        va_list ap;

        va_start(ap, format);
        __logn(format, ap);
        va_end(ap);
    }

    return 0;
}

int
err_print(const char *format, ...) {

    if (!app_quiet) {

        va_list ap;

        va_start(ap, format);
        __log(format, ap);
        va_end(ap);
    }

    return -1;
}

int
err_perror(const char *format, ...) {

    if (!app_quiet) {

        va_list ap;
        char buffer[1024];

        snprintf(buffer, sizeof(buffer),
                 "%s: %s", format, strerror(errno));

        va_start(ap, format);
        __log(buffer, ap);
        va_end(ap);
    }

    return -1;
}

/*
  Local variables: ***
  mode: c ***
  coding: iso-8859-15-unix ***
  c-basic-offset: 4 ***
  indent-tabs-mode: nil ***
  End: ***
*/
